==> sources_fancymenu_fabric_3.8.0_MC_1.19.2.jar: de/keksuccino/fancymenu/customization/element/elements/animationcontroller/AnimationControllerElement.java <==
package de.keksuccino.fancymenu.customization.element.elements.animationcontroller;

import com.mojang.blaze3d.systems.RenderSystem;
import de.keksuccino.fancymenu.customization.element.AbstractElement;
import de.keksuccino.fancymenu.customization.element.ElementBuilder;
import de.keksuccino.fancymenu.customization.layer.ScreenCustomizationLayer;
import de.keksuccino.fancymenu.customization.layer.ScreenCustomizationLayerHandler;
import de.keksuccino.fancymenu.util.rendering.gui.GuiGraphics;
import de.keksuccino.fancymenu.util.rendering.gui.GuiRenderTypes;
import org.jetbrains.annotations.NotNull;
import java.util.ArrayList;
import java.util.List;
import net.minecraft.class_310;

public class AnimationControllerElement extends AbstractElement {

    @NotNull
    public List<AnimationKeyframe> keyframes = new ArrayList<>();
    @NotNull
    public List<TargetElement> targetElements = new ArrayList<>();
    public boolean loop = false;
    public boolean offsetMode = false;
    public boolean ignoreSize = false;
    public boolean ignorePosition = false;

    public AnimationControllerElement(@NotNull ElementBuilder<?, ?> builder) {
        super(builder);
        this.allowDepthTestManipulation = true;
    }

    @Override
    public void render(@NotNull GuiGraphics graphics, int mouseX, int mouseY, float partial) {

        if (isEditor()) {

            if (this.shouldRender()) {

                int x = this.getAbsoluteX();
                int y = this.getAbsoluteY();
                int w = this.getAbsoluteWidth();
                int h = this.getAbsoluteHeight();
                RenderSystem.enableBlend();
                graphics.fill(x, y, x + w, y + h, this.inEditorColor.getColorInt());
                graphics.enableScissor(x, y, x + w, y + h);
                graphics.drawCenteredString(class_310.method_1551().field_1772, this.getDisplayName(), x + (w / 2), y + (h / 2) - (class_310.method_1551().field_1772.field_2000 / 2), -1);
                graphics.disableScissor();

            }

        } else {

            boolean shouldPlay = AnimationControllerStateController.isPlaying(this.getInstanceIdentifier());
            if (!shouldPlay) {
                AnimationControllerHandler.resetController(this);
                return;
            }

            this.targetElements.forEach(targetElement -> {
                if (this.shouldRender()) {
                    if (AnimationControllerHandler.wasAnimatedInThePast(targetElement.targetElementId) && AnimationControllerHandler.isFinished(targetElement.targetElementId) && !AnimationControllerHandler.isAnimating(targetElement.targetElementId)) {
                        targetElement.animationApplied = true;
                    } else {
                        ScreenCustomizationLayer layer = ScreenCustomizationLayerHandler.getActiveLayer();
                        if (layer != null) {
                            AbstractElement target = layer.getElementByInstanceIdentifier(targetElement.targetElementId);
                            if (target != null) targetElement.animationApplied = AnimationControllerHandler.applyAnimation(this, target);
                        }
                    }
                } else {
                    if (AnimationControllerHandler.wasAnimatedInThePast(targetElement.targetElementId) && !AnimationControllerHandler.isFinished(targetElement.targetElementId)) {
                        targetElement.animationApplied = false;
                    }
                }
            });

        }

    }

    public List<AnimationKeyframe> getKeyframes() {
        return new ArrayList<>(keyframes);
    }

    public static class TargetElement {

        public String targetElementId;
        public boolean animationApplied = false;

        public TargetElement() {
        }

        public TargetElement(String targetElementId) {
            this.targetElementId = targetElementId;
        }

    }

}

==> sources_fancymenu_fabric_3.8.1_MC_1.21.10.jar: de/keksuccino/fancymenu/customization/element/elements/animationcontroller/AnimationControllerElement.java <==
package de.keksuccino.fancymenu.customization.element.elements.animationcontroller;

import com.mojang.blaze3d.systems.RenderSystem;
import de.keksuccino.fancymenu.customization.element.AbstractElement;
import de.keksuccino.fancymenu.customization.element.ElementBuilder;
import de.keksuccino.fancymenu.customization.layer.ScreenCustomizationLayer;
import de.keksuccino.fancymenu.customization.layer.ScreenCustomizationLayerHandler;
import org.jetbrains.annotations.NotNull;
import java.util.ArrayList;
import java.util.List;
import net.minecraft.class_310;
import net.minecraft.class_332;

public class AnimationControllerElement extends AbstractElement {

    @NotNull
    public List<AnimationKeyframe> keyframes = new ArrayList<>();
    @NotNull
    public List<TargetElement> targetElements = new ArrayList<>();
    public boolean loop = false;
    public boolean offsetMode = false;
    public boolean ignoreSize = false;
    public boolean ignorePosition = false;

    public AnimationControllerElement(@NotNull ElementBuilder<?, ?> builder) {
        super(builder);
        this.allowDepthTestManipulation = true;
    }

    @Override
    public void method_25394(@NotNull class_332 graphics, int mouseX, int mouseY, float partial) {

        if (isEditor()) {

            if (this.shouldRender()) {

                int x = this.getAbsoluteX();
                int y = this.getAbsoluteY();
                int w = this.getAbsoluteWidth();
                int h = this.getAbsoluteHeight();
                graphics.method_25294(x, y, x + w, y + h, this.inEditorColor.getColorInt());
                graphics.method_44379(x, y, x + w, y + h);
                graphics.method_27534(class_310.method_1551().field_1772, this.getDisplayName(), x + (w / 2), y + (h / 2) - (class_310.method_1551().field_1772.field_2000 / 2), -1);
                graphics.method_44380();

            }

        } else {

            boolean shouldPlay = AnimationControllerStateController.isPlaying(this.getInstanceIdentifier());
            if (!shouldPlay) {
                AnimationControllerHandler.resetController(this);
                return;
            }

            this.targetElements.forEach(targetElement -> {
                if (this.shouldRender()) {
                    if (AnimationControllerHandler.wasAnimatedInThePast(targetElement.targetElementId) && AnimationControllerHandler.isFinished(targetElement.targetElementId) && !AnimationControllerHandler.isAnimating(targetElement.targetElementId)) {
                        targetElement.animationApplied = true;
                    } else {
                        ScreenCustomizationLayer layer = ScreenCustomizationLayerHandler.getActiveLayer();
                        if (layer != null) {
                            AbstractElement target = layer.getElementByInstanceIdentifier(targetElement.targetElementId);
                            if (target != null) targetElement.animationApplied = AnimationControllerHandler.applyAnimation(this, target);
                        }
                    }
                } else {
                    if (AnimationControllerHandler.wasAnimatedInThePast(targetElement.targetElementId) && !AnimationControllerHandler.isFinished(targetElement.targetElementId)) {
                        targetElement.animationApplied = false;
                    }
                }
            });

        }

    }

    public List<AnimationKeyframe> getKeyframes() {
        return new ArrayList<>(keyframes);
    }

    public static class TargetElement {

        public String targetElementId;
        public boolean animationApplied = false;

        public TargetElement() {
        }

        public TargetElement(String targetElementId) {
            this.targetElementId = targetElementId;
        }

    }

}
